@@ -28,32 +28,39 @@ Note:
         },
 """
 
+from __future__ import annotations
 import subprocess
 import shutil
 from pathlib import Path
-from . import paths
 import mylogging
+from typing import Union
 
 # Lazy imports
 # import EelForkExcludeFiles
 
+from . import paths
+from . import venvs
+from .paths import PROJECT_PATHS
+
 
 def build_app(
-    main_file="app.py",
-    preset=None,
-    web_path=None,
-    build_web=None,
-    remove_last_build=False,
-    console=True,
-    debug=False,
-    icon=None,
-    hidden_imports=[],
-    ignored_packages=[],
-    datas=[],
-    name=None,
-    env_vars={},
-    cleanit=True,
-):
+    root_path: Union[str, Path] = "infer",
+    main_file: Union[str, Path] = "app.py",
+    preset: str = None,
+    web_path: Union[str, Path] = "infer",
+    build_web: bool = True,
+    use_virutalenv: bool = True,
+    remove_last_build: bool = False,
+    console: bool = True,
+    debug: bool = False,
+    icon: Union[str, Path] = None,
+    hidden_imports: list[str] = [],
+    ignored_packages: list[str] = [],
+    datas: tuple[tuple[str, str], ...] = (),
+    name: str = None,
+    env_vars: dict = {},
+    cleanit: bool = True,
+) -> None:
     """One script to build .exe app from source code.
 
     This script automatically generate .spec file, build node web files and add environment variables during build.
@@ -62,12 +69,15 @@ def build_app(
     if you start with application.
 
     Args:
+        root_path (Union[str, Path], optional): Path of root folder where build and dist folders will be placed. Defaults to "infer".
         main_file (str, optional): Main file path or name with extension. Main file is found automatically
             and don't have to be in root. Defaults to 'app.py'.
         preset (str, optional): Edit other params for specific use cases (append to hidden_imports, datas etc.)
             Options ['eel']. Defaults to None.
-        web_path ((Path, str), optional): Folder with index.html. Defaults to None.
-        build_web (bool, optional): If application contain package.json in folder 'gui', build it (if using eel). Defaults to None.
+        web_path ((Path, str), optional): Folder with index.html. Defaults to 'infer'.
+        build_web ((bool, str), optional): If application contain package.json build node application. If 'preset' build automatically
+            depending on preset. Defaults to 'eel'.
+        use_virutalenv (bool, optional): Whether run new virtualenv and install all libraries from requirements.txt. Defaults to True.
         remove_last_build (bool, optional): If some problems, it is possible to delete build and dist folders. Defaults to False.
         console (bool, optional): Before app run terminal window appears (good for debugging). Defaults to False.
         debug (bool, optional): If no console, then dialog window with traceback appears. Defaults to False.
@@ -75,7 +85,7 @@ def build_app(
         hidden_imports (list, optional): If app is not working, it can be because some library was not builded. Add such
             libraries into this list. Defaults to [].
         ignored_packages (list, optional): Libraries take space even if not necessary. Defaults to [].
-        datas (list, optional): Add static files to build. Example: [('my_source_path, 'destination_path')]. Defaults to [].
+        datas (tuple[tuple[str, str], ...], optional): Add static files to build. Example: [('my_source_path, 'destination_path')]. Defaults to [].
         name (str, optional): If name of app is different than main py file. Defaults to None.
         env_vars (dict, optional): Add some env vars during build. Mostly to tell main script that it's production (ne development) mode.
             Defaults to {}.
@@ -92,22 +102,22 @@ def build_app(
         Back to pyinstaller folder and python `setup.py`
     """
 
-    if not paths.ROOT_PATH:
-        paths.set_root()
+    root_path = PROJECT_PATHS.ROOT_PATH if root_path == "infer" else paths.validate_path(root_path)
 
     # Try to recognize the structure of app
-    build_path = paths.ROOT_PATH / "build"
+    build_path = root_path / "build"
 
     if not build_path.exists():
         build_path.mkdir(parents=True, exist_ok=True)
 
     # Remove last dist manually to avoid permission error if opened in some application
-    dist_path = paths.ROOT_PATH / "dist"
+    dist_path = root_path / "dist"
 
     if dist_path.exists():
         try:
             shutil.rmtree(dist_path, ignore_errors=False)
         except (PermissionError, OSError):
+
             raise PermissionError(
                 mylogging.return_str(
                     "App is opened (May be in another app(terminal, explorer...)). Close it first."
@@ -120,7 +130,9 @@ def build_app(
     if not main_file_path.exists():
 
         # Iter paths and find the one
-        main_file_path = paths.find_path(main_file_path, exclude=["node_modules", "build"])
+        main_file_path = paths.find_path(
+            main_file_path.name,
+        )
 
         if not main_file_path.exists():
             raise KeyError("Main file not found, not infered and must be configured in params...")
@@ -138,7 +150,10 @@ def build_app(
         if not icon_path.exists():
 
             # Iter paths and find the one
-            icon_path = paths.find_path(icon_path, exclude=["node_modules", "build"])
+            icon_path = paths.find_path(
+                icon_path.name,
+                exclude_names=["node_modules", "build"],
+            )
 
             if not icon_path.exists():
                 raise KeyError("Icon not found, not infered check path or name...")
@@ -160,13 +175,27 @@ def build_app(
             pass
 
     # Build JS to static asset
-    if build_web:
+    if build_web is True or (build_web == "preset" and preset in ["eel"]):
         gui_path = paths.find_path("package.json").parent
-        subprocess.run(["npm", "run", "build"], shell=True, check=True, cwd=gui_path)
+        try:
+            builded = subprocess.run("npm run build", check=True, cwd=gui_path.as_posix(), shell=True)
+            if builded.returncode != 0:
+                raise RuntimeError()
+
+        except Exception:
+            mylogging.traceback(f"Build of web files failed. Try \n\nnpm run build\n\n in folder {gui_path}.")
+            raise
 
     if build_web or preset == "eel":
-        if not web_path:
-            web_path = paths.find_path("index.html", exclude=["public", "node_modules", "build"]).parent
+        if web_path == "infer":
+            web_path = paths.find_path(
+                "index.html",
+                exclude_names=[
+                    "public",
+                    "node_modules",
+                    "build",
+                ],
+            ).parent
 
         else:
             web_path = Path(web_path)
@@ -183,15 +212,22 @@ def build_app(
 
         import EelForkExcludeFiles
 
-        if build_web is None:
-            build_web = True
-
-        hidden_imports = [*hidden_imports, "EelForkExcludeFiles", "bottle_websocket"]
+        hidden_imports = [
+            *hidden_imports,
+            "EelForkExcludeFiles",
+            "bottle_websocket",
+        ]
         datas = (
             *datas,
-            (EelForkExcludeFiles._eel_js_file, "EelForkExcludeFiles"),
+            (
+                EelForkExcludeFiles._eel_js_file,
+                "EelForkExcludeFiles",
+            ),
         )
-        env_vars = {**env_vars, "MY_PYTHON_VUE_ENVIRONMENT": "production"}
+        env_vars = {
+            **env_vars,
+            "MY_PYTHON_VUE_ENVIRONMENT": "production",
+        }
 
     if env_vars:
         env_vars_template = f"""
@@ -260,13 +296,26 @@ coll = COLLECT(exe,
         spec_file.write(spec_template)
 
     # Build py to exe
+    command_list = ["pyinstaller", "-y", spec_path.as_posix()]
 
-    subprocess.run(
-        ["pyinstaller", "-y", spec_path.as_posix()],
-        shell=True,
-        check=True,
-        cwd=paths.ROOT_PATH,
-    )
+    if use_virutalenv:
+        my_venv = venvs.MyVenv(PROJECT_PATHS.ROOT_PATH / "venv")
+        my_venv.create()
+        my_venv.sync_requirements()
+
+        command_list = [*my_venv.activate_command.split(), " && ", *command_list]
+
+    try:
+        subprocess.run(" ".join(command_list), check=True, cwd=PROJECT_PATHS.ROOT_PATH.as_posix(), shell=True)
+
+    except (Exception,):
+        mylogging.traceback(
+            "Build with pyinstaller failed. First, check if `pyinstaller` is installed. Check it with pip list in used python interpreter. "
+            f" Try (if windows, use cmd) \n\n{' '.join(command_list)}\n\n in folder `{PROJECT_PATHS.ROOT_PATH.as_posix()}`.\n\n"
+            "Troubleshooting: If there are still errors, try to install newset pyinstaller locally with `python setup.py install`, "
+            "update setuptools, delete `build` and `dist` folder and try again."
+        )
+        raise
 
     if cleanit:
         try:
