@@ -1,5 +1,9 @@
-"""Plot data. There is only one main function plot. Check it's documentation for how to use it.
-"""
+"""Plot data. There is only one main function plot. Check it's documentation for how to use it."""
+
+from __future__ import annotations
+from typing import Union
+
+from typing_extensions import Literal
 
 import mylogging
 
@@ -16,22 +20,22 @@ from . import paths
 
 def plot(
     complete_dataframe,
-    plot_type="plotly",
-    plot_name="Plot",
-    legend=True,
+    plot_library: Literal["plotly", "matplotlib"] = "plotly",
+    plot_name: str = "Plot",
+    legend: bool = True,
     highlighted_column="",
     surrounded_column="",
-    grey_area=False,
-    save=False,
-    plot_return=False,
-    show=True,
-):
+    grey_area: bool = False,
+    save: bool = False,
+    return_div: bool = False,
+    show: bool = True,
+) -> Union[None, str]:
     """Plots the data. Plotly or matplotlib can be used. It is possible to highlite two columns with different formating.
     It is usually used for time series visualization, but it can be used for different use case of course.
 
     Args:
         complete_dataframe (pd.DataFrame): Data to be plotted.
-        plot_type (str, optional): 'plotly' or 'matplotlib'. Defaults to "plotly".
+        plot_library (Literal['plotly', 'matplotlib'], optional): 'plotly' or 'matplotlib'. Defaults to "plotly".
         legend (bool, optional): Whether display legend or not. Defaults to True.
         highlighted_column (str, optional): Column name that will be formatted differently (blue). Can be empty. Defaults to "".
         surrounded_column (str, optional): Column name that will be formatted differently (black, wider). And is surrounded by
@@ -40,25 +44,29 @@ def plot(
             or list of ['lower_bound_column', 'upper_bound_column']. Both columns has to be in complete_dataframe. Defaults to False.
         save ((False, str), optional): Whether save the plot.  If False or "", do not save, if path as str, save to defined path,
             if "DESKTOP" save to desktop. Defaults to "".
-        plot_return ((bool, str), optional): If 'div', return html div with plot as string. If False, just plot and do not return.
+        return_div ((bool, str), optional): If 'div', return html div with plot as string. If False, just plot and do not return.
             Defaults to False.
         show (bool, optional): Can be evaluated, but not shown (testing reasons). Defaults to True.
 
     Returns:
-        str: Only if plot_return == 'div
+        str: Only if return_div is True, else None.
 
     Examples:
         Plot dataframe with
 
         >>> import pandas as pd
-        >>> plot(pd.DataFrame([[None, None, 1], [None, None, 2], [3, 3, 6], [3, 2.5, 4]]), show=False)  # Show False just for testing reasons
+        >>> df = pd.DataFrame([[None, None, 1], [None, None, 2], [3, 3, 6], [3, 2.5, 4]])
+        >>> plot(df, show=False)  # Show False just for testing reasons
 
+        You can use matplotlib
+
+        >>> plot(df, plot_library="matplotlib")
     """
 
     if save == "DESKTOP":
         save = paths.get_desktop_path() / "plot.html"
 
-    if plot_type == "matplotlib":
+    if plot_library == "matplotlib":
         if misc._JUPYTER:
             from IPython import get_ipython
 
@@ -84,7 +92,7 @@ def plot(
         if show:
             plt.show()
 
-    elif plot_type == "plotly":
+    elif plot_library == "plotly":
 
         import plotly as pl
 
@@ -116,7 +124,10 @@ def plot(
                     name=surrounded_column,
                     x=complete_dataframe.index,
                     y=complete_dataframe[surrounded_column],
-                    line={"color": "rgb(51, 19, 10)", "width": 5},
+                    line={
+                        "color": "rgb(51, 19, 10)",
+                        "width": 5,
+                    },
                     fillcolor="rgba(68, 68, 68, 0.3)",
                     fill="tonexty" if grey_area else None,
                 )
@@ -152,7 +163,10 @@ def plot(
                     name=str(highlighted_column),
                     x=complete_dataframe.index,
                     y=complete_dataframe[highlighted_column],
-                    line={"color": "rgb(31, 119, 180)", "width": 2},
+                    line={
+                        "color": "rgb(31, 119, 180)",
+                        "width": 2,
+                    },
                 )
 
                 used_columns.remove(highlighted_column)
@@ -170,7 +184,11 @@ def plot(
         for i in complete_dataframe.columns:
             if i in used_columns:
                 fig.add_trace(
-                    pl.graph_objs.Scatter(x=complete_dataframe.index, y=complete_dataframe[i], name=i)
+                    pl.graph_objs.Scatter(
+                        x=complete_dataframe.index,
+                        y=complete_dataframe[i],
+                        name=i,
+                    )
                 )
 
         fig.layout.update(
@@ -187,7 +205,12 @@ def plot(
             legend_orientation="h",
             hoverlabel={"namelength": -1},
             font={"size": 17},
-            margin={"l": 160, "r": 130, "b": 160, "t": 110},
+            margin={
+                "l": 160,
+                "r": 130,
+                "b": 160,
+                "t": 110,
+            },
         )
 
         if show:
@@ -196,7 +219,7 @@ def plot(
         if save:
             fig.write_html(save)
 
-        if plot_return == "div":
+        if return_div == "div":
 
             fig.layout.update(
                 title=None,
@@ -205,4 +228,8 @@ def plot(
                 margin={"b": 35, "t": 35, "pad": 4},
             )
 
-            return pl.offline.plot(fig, include_plotlyjs=False, output_type="div")
+            return pl.offline.plot(
+                fig,
+                include_plotlyjs=False,
+                output_type="div",
+            )
