@@ -19,11 +19,13 @@ Image of such an app
 
 """
 
+from __future__ import annotations
+from typing import TYPE_CHECKING
 import os
 import sys
 from pathlib import Path
 import warnings
-
+from typing import Union, Callable
 
 import mylogging
 
@@ -31,12 +33,12 @@ from . import paths
 from . import misc
 
 # Lazy imports
-# import numpy as np
-# import inspect
-# import ast
+if TYPE_CHECKING:
+    import numpy as np
+    import pandas as pd
 
-# import pandas as pd
-# import EelForkExcludeFiles as eel
+    # import mydatapreprocessing as mdp
+    # import EelForkExcludeFiles as eel
 
 
 eel = None
@@ -45,7 +47,12 @@ expose_error_callback = None
 json_to_py = misc.json_to_py
 
 
-def run_gui(devel=None, log_file_path=None, is_multiprocessing=False, builded_gui_path="default"):
+def run_gui(
+    devel: Union[bool, None] = None,
+    log_file_path: Union[str, Path] = None,
+    is_multiprocessing: bool = False,
+    build_gui_path: Union[str, Path] = "default",
+) -> None:
     """Function that init and run `eel` project.
 
     It will autosetup chrome mode (if installed chrome or chromium, open separate window with
@@ -53,7 +60,8 @@ def run_gui(devel=None, log_file_path=None, is_multiprocessing=False, builded_gu
     on windows).
 
     In devel mode, app is connected on live vue server. Serve your web application with node, debug python app file
-    that calls this function (do not run, just debug - server could stay running after close and occupy used port). Open browser on 8080.
+    that calls this function (do not run, just debug - server could stay running after close and occupy used port).
+    Open browser on 8080.
 
     Debugger should correctly stop at breakpoints if frontend run some python function.
 
@@ -63,12 +71,12 @@ def run_gui(devel=None, log_file_path=None, is_multiprocessing=False, builded_gu
         https://mypythontools.readthedocs.io/#project-starter
 
     Args:
-        devel((bool, None), optional): If None, detected. Can be overwritten. Devel 0 run static assets, 1 run Vue server on localhost.
-            Defaults to None.
+        devel((bool, None), optional): If None, detected. Can be overwritten. Devel 0 run static assets,
+            1 run Vue server on localhost. Defaults to None.
         log_file_path ((str, Path, None)), optional): If not exist, it will create, if exist, it will append,
             if None, log to relative log.log and only if in production mode. Defaults to None.
         is_multiprocessing (bool, optional): If using multiprocessing in some library, set up to True. Defaults to False.
-        builded_gui_path ((str, Path, None)), optional): Where the web asset is. Only if debug is 0 but not run with pyinstaller.
+        build_gui_path ((str, Path, None)), optional): Where the web asset is. Only if debug is 0 but not run with pyinstaller.
             If None, it's automatically find (but is slower then). If 'default', path from project-starter is used - 'gui/web_builded'
             is used. Defaults to 'default'.
 
@@ -81,7 +89,11 @@ def run_gui(devel=None, log_file_path=None, is_multiprocessing=False, builded_gu
     global eel
 
     with warnings.catch_warnings():
-        warnings.filterwarnings("ignore", module="EelForkExcludeFiles", category=ResourceWarning)
+        warnings.filterwarnings(
+            "ignore",
+            module="EelForkExcludeFiles",
+            category=ResourceWarning,
+        )
 
         import EelForkExcludeFiles as eel_library
 
@@ -93,37 +105,45 @@ def run_gui(devel=None, log_file_path=None, is_multiprocessing=False, builded_gu
             devel = False if os.environ.get("MY_PYTHON_VUE_ENVIRONMENT") == "production" else True
 
         # Whether run is from .exe or from python
-        is_builded = True if getattr(sys, "frozen", False) else False
+        is_built = True if getattr(sys, "frozen", False) else False
 
         if log_file_path:
             log_file = log_file_path
         else:
-            log_file = "log.log" if is_builded else None
+            log_file = "log.log" if is_built else None
 
-        mylogging.config.TO_FILE = log_file
+        mylogging.config.OUTPUT = log_file
 
-        if is_builded:
+        if is_built:
             # gui folder is created with pyinstaller in build
             gui_path = Path(sys._MEIPASS) / "gui"
         else:
             if devel:
-                paths.set_root()
                 gui_path = (
-                    paths.find_path("index.html", exclude=["node_modules", "build", "dist"]).parents[1]
+                    paths.find_path(
+                        "index.html",
+                    ).parents[1]
                     / "src"
                 )
             else:
-                if builded_gui_path:
-                    gui_path = Path(builded_gui_path)
+                if build_gui_path:
+                    gui_path = Path(build_gui_path)
 
                 else:
-                    paths.set_root()
                     gui_path = paths.find_path(
-                        "index.html", exclude=["public", "node_modules", "build"]
+                        "index.html",
+                        exclude_names=[
+                            "public",
+                            "node_modules",
+                            "build",
+                            "dist",
+                        ],
                     ).parent
 
         if not gui_path.exists():
-            raise FileNotFoundError("Web files not found, setup gui_path (where builded index.html is).")
+            raise FileNotFoundError(
+                "Web files not found, setup `build_gui_path` (where builded index.html is)."
+            )
 
         if devel:
             directory = gui_path
@@ -143,7 +163,11 @@ def run_gui(devel=None, log_file_path=None, is_multiprocessing=False, builded_gu
             port = 0
             init_files = [".js", ".html"]
 
-        eel.init(directory.as_posix(), init_files, exlcude_patterns=["chunk-vendors"])
+        eel.init(
+            directory.as_posix(),
+            init_files,
+            exlcude_patterns=["chunk-vendors"],
+        )
 
         if is_multiprocessing:
             from multiprocessing import freeze_support
@@ -174,9 +198,10 @@ def run_gui(devel=None, log_file_path=None, is_multiprocessing=False, builded_gu
 
     except Exception:
         mylogging.traceback("Py side terminated...")
+        raise
 
 
-def expose(callback_function):
+def expose(callback_function: Callable) -> None:
     """Wrap eel expose with try catch block and adding exception callback function
     (for printing error to frontend usually).
 
@@ -185,7 +210,11 @@ def expose(callback_function):
     """
 
     with warnings.catch_warnings():
-        warnings.filterwarnings("ignore", module="EelForkExcludeFiles", category=ResourceWarning)
+        warnings.filterwarnings(
+            "ignore",
+            module="EelForkExcludeFiles",
+            category=ResourceWarning,
+        )
 
         import EelForkExcludeFiles as eel
 
@@ -202,7 +231,7 @@ def expose(callback_function):
     eel._expose(callback_function.__name__, inner)
 
 
-def to_vue_plotly(data, names=None):
+def to_vue_plotly(data: Union["np.ndarray", "pd.DataFrame"], names: list = None) -> dict:
     """Takes data (dataframe or numpy array) and transforms it to form, that vue-plotly understand.
 
     Links to vue-plotly:
@@ -215,33 +244,56 @@ def to_vue_plotly(data, names=None):
         Download the js function from project-starter and check for example.
 
     Args:
-        data ((np.array, pd.DataFrame)): Plotted data.
-        names (list, optional): If using array, you can define names. Defaults to None.
+        data (Union[np.array, pd.DataFrame]): Plotted data.
+        names (list, optional): If using array, you can define names. If using pandas, columns are
+            automatically used. Defaults to None.
 
     Returns:
         dict: Data in form for plotting in frontend.
-    """
 
+    Example:
+        >>> import pandas as pd
+        >>> df = pd.DataFrame([[1, "a"], [2, "b"]], columns=["numbers", "letters"])
+        >>> to_vue_plotly(df)
+        {'x_axis': [0, 1], 'y_axis': [[1.0, 2.0]], 'names': ['numbers']}
+    """
     import pandas as pd
     import numpy as np
 
-    if isinstance(data, (np.ndarray, np.generic)):
+    import mydatapreprocessing as mdp
+
+    if isinstance(data, np.ndarray):
         data = pd.DataFrame(data, columns=names)
 
     data = pd.DataFrame(data)
 
     numeric_data = data.select_dtypes(include="number").round(decimals=3)
-    numeric_data = numeric_data.where(np.isfinite(numeric_data), None)
-    # numeric_data = add_none_to_gaps(numeric_data)
 
+    # TODO fix datetime
+    try:
+        numeric_data = mdp.misc.add_none_to_gaps(numeric_data)
+    except Exception:
+        pass
+
+    numeric_data = numeric_data.where(np.isfinite(numeric_data), np.nan)
+
+    # TODO
+    # Remove dirty hack... editing lists
+
+    values_list = numeric_data.values.T.tolist()
+
+    for i, j in enumerate(values_list):
+        values_list[i] = [k if not np.isnan(k) else None for k in j]
+
+    # TODO use typed dict
     return {
         "x_axis": numeric_data.index.to_list(),
-        "y_axis": numeric_data.values.T.tolist(),
+        "y_axis": values_list,
         "names": numeric_data.columns.values.tolist(),
     }
 
 
-def to_table(df, index=False):
+def to_table(df: "pd.DataFrame", index: bool = False) -> dict:
     """Takes data (dataframe or numpy array) and transforms it to form, that vue-plotly library understands.
 
     Args:
@@ -250,6 +302,12 @@ def to_table(df, index=False):
 
     Returns:
         dict: DatPa in form for creating table in Vuetify v-data-table.
+
+    Example:
+        >>> import pandas as pd
+        >>> df = pd.DataFrame([[1, "a"], [2, "b"]], columns=["numbers", "letters"])
+        >>> to_table(df)
+        {'table': [{'numbers': 1, 'letters': 'a'}, {'numbers': 2, 'letters': 'b'}], 'headers': [{'text': 'numbers', 'value': 'numbers', 'sortable': True}, {'text': 'letters', 'value': 'letters', 'sortable': True}]}
     """
     import pandas as pd
     import numpy as np
@@ -274,4 +332,8 @@ def to_table(df, index=False):
 
     headers = [{"text": i, "value": i, "sortable": True} for i in data.columns]
 
-    return {"table": data.to_dict("records"), "headers": headers}
+    # TODO use typed dict
+    return {
+        "table": data.to_dict("records"),
+        "headers": headers,
+    }
