@@ -4,8 +4,7 @@ This module can be used for example in running deploy pipelines or githooks
 edit library version, generate rst files for docs, push to git or deploy app to pypi.
 
 All of that can be done with one function call - with `push_pipeline` function that
-run other functions, or you can use functions separately. If you are using other
-function than `push_pipeline`, you need to call `mypythontools.paths.set_paths()` first.
+run other functions, or you can use functions separately. 
 
 
 Examples:
@@ -18,14 +17,14 @@ Examples:
     be configured on the beginning and therefore you don't need to wait for test finish.
     Default values can be also used, so in small starting projects, push is actually very fast.
 
-    Create folder utils, create `push_script.py` inside, add
+    Create folder utils, create `push_script.py` inside, add::
 
-    >>> import mypythontools
-    ...
-    >>> if __name__ == "__main__":
-    ...     # Params that are always the same define here. Params that are changing define in IDE when run action.
-    ...     # For example in tasks (command line arguments and argparse will be used).
-    ...     mypythontools.utils.push_pipeline(deploy=True)
+        import mypythontools
+
+        if __name__ == "__main__":
+            # Params that are always the same define here. Params that are changing define in IDE when run action.
+            # For example in tasks (command line arguments and argparse will be used).
+            mypythontools.utils.push_pipeline(deploy=True)
 
     Then just add this task to global tasks.json::
 
@@ -107,63 +106,68 @@ Examples:
         # -*- coding: UTF-8 -*-
 
     Then just import any function from here and call with desired params. E.g.
-
-    >>> import mypythontools
-    >>> mypythontools.paths.set_paths()
-    >>> version = mypythontools.utils.get_version()  # For example 1.0.2
-    >>> print(version[0].isdigit() == version[2].isdigit() == version[5].isdigit() == (version[1] == '.' == version[3]))
-    True
 """
 
+from __future__ import annotations
 import argparse
 import ast
-import importlib
+import importlib.util
 from pathlib import Path
 import subprocess
 import sys
+from typing import Union
 
 import mylogging
 
-from . import paths
+from . import misc
 from . import tests
 from . import deploy as deploy_module
+from .paths import PROJECT_PATHS, validate_path
 
 # Lazy loaded
 # from git import Repo
+# import json
 
 
 def push_pipeline(
-    test=True,
-    test_options={},
-    version="increment",
-    sphinx_docs=True,
-    push_git=True,
-    commit_message="New commit",
-    tag="__version__",
-    tag_mesage="New version",
-    deploy=False,
-):
+    reformat: bool = True,
+    test: bool = True,
+    test_options: dict = {},
+    version: str = "increment",
+    sphinx_docs: Union[bool, list[str]] = True,
+    push_git: bool = True,
+    commit_message: str = "New commit",
+    tag: str = "__version__",
+    tag_mesage: str = "New version",
+    deploy: bool = False,
+) -> None:
     """Run pipeline for pushing and deploying app. Can run tests, generate rst files for sphinx docs,
     push to github and deploy to pypi. All params can be configured not only with function params,
     but also from command line with params and therefore callable from terminal and optimal to run
     from IDE (for example with creating simple VS Code task).
 
+    Some function suppose some project structure (where are the docs, where is __init__.py etc.).
+    If you are issuing some error, try functions directly, find necessary paths in parameters
+    and set paths that are necessary in paths module.
+
     Check utils module docs for implementation example.
 
     Args:
+        reformat (bool, optional): Reformat all python files with black. Setup parameters in `pyproject.toml`,
+            especially setup `line-length`. Defaults to True.
         test (bool, optional): Whether run pytest tests. Defaults to True.
-        test_options (dict, optional): Parameters of tests function e.g. `{"test_coverage": True, "verbose": False, "use_virutalenv":True}`.
-            Defaults to {}.
+        test_options (dict, optional): Parameters of tests function e.g.
+            `{"test_coverage": True, "verbose": False, "use_virutalenv":True}`. Defaults to {}.
         version (str, optional): New version. E.g. '1.2.5'. If 'increment', than it's auto incremented. E.g from '1.0.2' to 'v1.0.3'.
             If None, then version is not changed. 'Defaults to "increment".
-        sphinx_docs((bool, list), optional): Whether generate sphinx apidoc and generate rst files for documentation.
+        sphinx_docs(Union[bool, list[str]], optional): Whether generate sphinx apidoc and generate rst files for documentation.
             Some files in docs source can be deleted - check `sphinx_docs` docstrings for details and insert
             `exclude_paths` list if have some extra files other than ['conf.py', 'index.rst', '_static', '_templates'].
             Defaults to True.
         push_git (bool, optional): Whether push repository on git with git_message, tag and tag message. Defaults to True.
         git_message (str, optional): Git message. Defaults to 'New commit'.
-        tag (str, optional): Used tag. If None, not tag will be pushed. If tag is '__version__', than updated version from __init__ is used.
-            Defaults to __version__.
+        tag (str, optional): Used tag. If None, not tag will be pushed. If tag is '__version__', than updated
+            version from __init__ is used. Defaults to __version__.
         tag_mesage (str, optional): Tag message. Defaults to New version.
         deploy (bool, optional): Whether deploy to PYPI. Defaults to False.
 
@@ -171,13 +175,14 @@ def push_pipeline(
         Recommended use is from IDE (for example with Tasks in VS Code). Check utils docs for how to use it.
         You can also use it from python...
 
-        >>> import mypythontools
-        ...
-        >>> if __name__ == "__main__":
-        ...     mypythontools.utils.push_pipeline(deploy=True)  # All the params that change everytime configure for example in VS Code tasks.
+        Put it in `if __name__ == "__main__":` block
+
+        >>> push_pipeline(push_git=False, deploy=False)
 
     """
+
     config = {
+        "reformat": reformat,
         "test": test,
         "test_options": test_options,
         "version": version,
@@ -189,60 +194,153 @@ def push_pipeline(
         "deploy": deploy,
     }
 
-    if not all([paths.ROOT_PATH, paths.APP_PATH, paths.INIT_PATH]):
-        paths.set_paths()
-
     if len(sys.argv) > 1:
         parser = argparse.ArgumentParser(description="Prediction framework setting via command line parser!")
 
+        parser.add_argument(
+            "--reformat",
+            type=bool,
+            help=(
+                "Whether reformat all python files with black. Setup parameters in pyproject.toml. Defaults to True."
+            ),
+        )
+        parser.add_argument(
+            "--test",
+            type=bool,
+            help=("Whether run pytest. Defaults to: True."),
+        )
+        parser.add_argument(
+            "--test_options",
+            type=str,
+            help=(
+                "Check tests module and function run_tests for what parameters you can use. Defaults to: {}."
+            ),
+        )
         parser.add_argument(
             "--version",
             type=str,
             help=(
                 "Version in __init__.py will be overwiten. Version has to be in format like '1.0.3' three digits"
                 "and two dots. If None, nothing will happen. If 'increment', than it will be updated by 0.0.1."
+                "Defaults to: 'increment'."
             ),
         )
+        parser.add_argument(
+            "--sphinx_docs",
+            type=bool,
+            help=("Whether run apidoc to create files for example for readthedocs. Defaults to: True."),
+        )
+        parser.add_argument(
+            "--push_git",
+            type=str,
+            help=("Whether push to github or not. Defaults to: True."),
+        )
         parser.add_argument(
             "--commit_message",
             type=str,
-            help="Commit message. Defaults to: 'New commit'",
+            help="Commit message. Defaults to: 'New commit'.",
         )
         parser.add_argument(
             "--tag",
             type=str,
-            help="Tag. E.g 'v1.1.2'. If '__version__', get the version. Defaults to: '__version__'",
+            help="Tag. E.g 'v1.1.2'. If '__version__', get the version. Defaults to: '__version__'.",
+        )
+        parser.add_argument(
+            "--tag_mesage",
+            type=str,
+            help="Tag message. Defaults to: 'New version'.",
+        )
+        parser.add_argument(
+            "--deploy",
+            type=bool,
+            help=(
+                "Whether deploy to pypi. `TWINE_USERNAME` and `TWINE_PASSWORD` are used for authorization."
+                "Defaults to: False."
+            ),
         )
-        parser.add_argument("--tag_mesage", type=str, help="Tag message. Defaults to: 'New version'")
-
         parser_args_dict = {k: v for k, v in parser.parse_known_args()[0].__dict__.items() if v is not None}
 
         if parser_args_dict:
             config.update(parser_args_dict)
 
+    if config["reformat"]:
+        reformat_with_black()
+
     if config["test"]:
+        if isinstance(config["test_options"], str):
+            import json
+
+            config["test_options"] = json.loads(config["test_options"])
+
         tests.run_tests(**test_options)
 
     if config["version"]:
+        original_version = get_version()
         set_version(config["version"])
 
-    if isinstance(config["sphinx_docs"], list):
-        sphinx_docs_regenerate(exclude_paths=config["sphinx_docs"])
-    elif config["sphinx_docs"]:
-        sphinx_docs_regenerate()
+    try:
+        if isinstance(config["sphinx_docs"], list):
+            sphinx_docs_regenerate(exclude_paths=config["sphinx_docs"])
+        elif config["sphinx_docs"]:
+            sphinx_docs_regenerate()
 
-    if push_git:
-        git_push(
-            commit_message=config["commit_message"],
-            tag=config["tag"],
-            tag_message=config["tag_mesage"],
+        if push_git:
+            git_push(
+                commit_message=config["commit_message"],
+                tag=config["tag"],
+                tag_message=config["tag_mesage"],
+            )
+    except Exception:
+        if config["version"]:
+            set_version(original_version)
+
+        mylogging.traceback(
+            "Utils pipeline failed. Original version restored. Nothing was pushed to repo, you can restart pipeline."
+        )
+        return
+
+    try:
+        if config["deploy"]:
+            deploy_module.deploy_to_pypi()
+    except Exception:
+        mylogging.traceback(
+            "Deploy failed, but pushed to repository already. Deploy manually. Version already changed.",
+            level="FATAL",
         )
 
-    if config["deploy"]:
-        deploy_module.deploy_to_pypi()
+
+def reformat_with_black(root_path: Union[str, Path] = "infer", extra_args: list[str] = ["--quiet"]) -> None:
+    """Reformat code with black.
+
+    Args:
+        root_path (Union[str, Path], optional): Root path of project. Defaults to "infer".
+        extra_args (list[str], optional): Some extra args for black. Defaults to ["--quiet"].
+
+    Example:
+        >>> reformat_with_black()
+    """
+    root_path = PROJECT_PATHS.ROOT_PATH if root_path == "infer" else validate_path(root_path)
+
+    try:
+        subprocess.run(f"black . {' '.join(extra_args)}", check=True, cwd=root_path)
+    except FileNotFoundError:
+        mylogging.traceback(
+            "FileNotFoundError can happen if `black` is not installed. Check it with pip list in used python interpreter."
+        )
+        raise
+    except (Exception,):
+        mylogging.traceback(
+            "Reformatting with `black` failed. Check if it's installed, check logged error, "
+            "then try format manually with \n\nblack .\n\n"
+        )
+        raise
 
 
-def git_push(commit_message, tag="__version__", tag_message="New version"):
+def git_push(
+    commit_message: str,
+    tag: str = "__version__",
+    tag_message: str = "New version",
+) -> None:
     """Stage all changes, commit, add tag and push. If tag = '__version__', than tag
     is infered from __init__.py.
 
@@ -253,52 +351,54 @@ def git_push(commit_message, tag="__version__", tag_message="New version"):
         tag_message (str, optional): Message in anotated tag. Defaults to 'New version'.
     """
 
-    from git import Repo
+    import git.repo
 
-    git_add_command = "git add . "
-
-    subprocess.run(git_add_command.split(), shell=True, check=True, cwd=paths.ROOT_PATH)
-
-    subprocess.run(
-        ["git", "commit", "-m", commit_message],
-        shell=True,
-        check=True,
-        cwd=paths.ROOT_PATH,
-    )
-
-    if not tag_message:
-        tag_message = "New version"
+    git_command = f"git add . && git commit -m {misc.get_console_str_with_quotes(commit_message)} && git push"
 
     if tag == "__version__":
         tag = f"v{get_version()}"
 
-    git_push_command = "git push"
-
     if tag:
-        Repo(paths.ROOT_PATH).create_tag(tag, message=tag_message)
-        git_push_command += " --follow-tags"
+        if not tag_message:
+            tag_message = "New version"
 
-    subprocess.run(git_push_command, shell=True, check=True, cwd=paths.ROOT_PATH)
+        git.repo.Repo(PROJECT_PATHS.ROOT_PATH.as_posix()).create_tag(tag, message=tag_message)
+        git_command += " --follow-tags"
+
+    try:
+        subprocess.run(git_command, check=True, cwd=PROJECT_PATHS.ROOT_PATH.as_posix(), shell=True)
+    except (Exception,):
+        git.repo.Repo(PROJECT_PATHS.ROOT_PATH.as_posix()).delete_tag(tag)
+        mylogging.traceback(
+            "Push to git failed. Version restored and created git tag deleted."
+            f"Try to run command \n\n{git_command}\n\n manually in your root {PROJECT_PATHS.ROOT_PATH}."
+        )
+        raise
 
 
-def set_version(version="increment"):
+def set_version(
+    version: str = "increment",
+    init_path: Union[str, Path] = "infer",
+) -> None:
     """Change your version in your __init__.py file.
 
 
     Args:
-        version (str, optional): If version is 'increment', it will increment your __version__
-            in you __init__.py by 0.0.1. Defaults to "increment".
+        version (str, optional): Form that is used in __init__, so for example "1.2.3". Do not use 'v' appendix.
+            If version is 'increment', it will increment your __version__ in you __init__.py by 0.0.1. Defaults to "increment".
+        init_path (Union[str, Path], optional): Path of file where __version__ is defined. Usually __init__.py, Defaults to "infer".
 
     Raises:
-        ValueError: If no __version__ is find. Try set INIT_PATH via paths.set_paths...
+        ValueError: If no __version__ is find.
     """
 
+    init_path = PROJECT_PATHS.INIT_PATH if init_path == "infer" else validate_path(init_path)
+
     if version == "increment" or (
-        len(version) == 5
-        and version[1] == version[3] == "."
-        and (version[0].isdigit() and version[2].isdigit() and version[4].isdigit())
+        len(version.split(".")) == 3 and all([i.isdecimal() for i in version.split(".")])
     ):
         pass
+
     else:
         raise ValueError(
             mylogging.return_str(
@@ -306,7 +406,7 @@ def set_version(version="increment"):
             )
         )
 
-    with open(paths.INIT_PATH, "r") as init_file:
+    with open(init_path.as_posix(), "r") as init_file:
 
         list_of_lines = init_file.readlines()
 
@@ -334,29 +434,32 @@ def set_version(version="increment"):
                 mylogging.return_str("__version__ variable not found in __init__.py. Try set INIT_PATH.")
             )
 
-    with open(paths.INIT_PATH, "w") as init_file:
+    with open(init_path.as_posix(), "w") as init_file:
 
         init_file.writelines(list_of_lines)
 
 
-def get_version(INIT_PATH=None):
+def get_version(init_path: Union[str, Path] = "infer") -> str:
     """Get version info from __init__.py file.
 
     Args:
-        INIT_PATH ((str, Path), optional): Path to __init__.py file. If None, it's taken from paths module
-            if used paths.set_paths() before. Defaults to None.
+        init_path (Union[str, Path], optional): Path to __init__.py file. Defaults to "infer".
 
     Returns:
         str: String of version from __init__.py.
 
     Raises:
-        ValueError: If no __version__ is find. Try set INIT_PATH...
+        ValueError: If no __version__ is find. Try set init_path...
+
+    Example:
+        >>> version = get_version()
+        >>> len(version.split(".")) == 3 and all([i.isdecimal() for i in version.split(".")])
+        True
     """
 
-    if not INIT_PATH:
-        INIT_PATH = paths.INIT_PATH
+    init_path = PROJECT_PATHS.INIT_PATH if init_path == "infer" else validate_path(init_path)
 
-    with open(INIT_PATH, "r") as init_file:
+    with open(init_path.as_posix(), "r") as init_file:
 
         for line in init_file:
 
@@ -365,14 +468,16 @@ def get_version(INIT_PATH=None):
                 return line.split(delim)[1]
 
         else:
-            raise ValueError(
-                mylogging.return_str("__version__ variable not found in __init__.py. Try set INIT_PATH.")
-            )
+            raise ValueError(mylogging.return_str("__version__ variable not found in __init__.py."))
 
 
 def sphinx_docs_regenerate(
-    docs_path=None, build_locally=False, git_add=True, exclude_paths=[], delete=["modules.rst"]
-):
+    docs_path: Union[str, Path] = "infer",
+    build_locally: bool = False,
+    git_add: bool = True,
+    exclude_paths: list[Union[str, Path]] = [],
+    delete: list[Union[str, Path]] = ["modules.rst"],
+) -> None:
     """This will generate all rst files necessary for sphinx documentation generation with sphinx-apidoc.
     It automatically delete removed and renamed files.
 
@@ -384,15 +489,15 @@ def sphinx_docs_regenerate(
     Function suppose sphinx build and source in separate folders...
 
     Args:
-        docs_path ((str, Path), optional): Where source folder is. Usually infered automatically.
-            Defaults to None.
+        docs_path (Union[str, Path], optional): Where source folder is. Usually infered automatically.
+            Defaults to "infer".
         build_locally (bool, optional): If true, build folder with html files locally.
             Defaults to False.
         git_add (bool, optional): Whether to add generated files to stage. False mostly for
             testing reasons. Defaults to True.
-        exclude_paths (list, optional): List of files and folder names that will not be deleted.
+        exclude_paths (list[Union[str, Path]], optional): List of files and folder names that will not be deleted.
             ['conf.py', 'index.rst', '_static', '_templates'] are excluded by default. Defaults to [].
-        delete (list, optional): If delete some files (for example to have no errors in sphinx build for unused modules)
+        delete (list[Union[str, Path]], optional): If delete some files (for example to have no errors in sphinx build for unused modules)
 
     Note:
         Function suppose structure of docs like::
@@ -401,29 +506,16 @@ def sphinx_docs_regenerate(
             -- -- source
             -- -- -- conf.py
             -- -- make.bat
-
-        If you are issuing error, try set project root path with `set_root`
     """
 
     if not importlib.util.find_spec("sphinx"):
         raise ImportError(
             mylogging.return_str(
-                "Sphinx library is necessary for docs generation. Install via `pip install sphinx`"
+                "Sphinx library is necessary for docs generation. Install via \n\npip install sphinx\n\n"
             )
         )
 
-    if not docs_path:
-        if paths.ROOT_PATH:
-            docs_path = paths.ROOT_PATH / "docs"
-        else:
-            raise NotADirectoryError(
-                mylogging.return_str(
-                    "`docs_path` not found. Setup it with parameter `docs_path` or use `paths.set_paths()` function."
-                )
-            )
-
-    if not all([paths.APP_PATH, paths.ROOT_PATH]):
-        mylogging.return_str("Paths are not known. First run `paths.set_paths()`.")
+    docs_path = PROJECT_PATHS.DOCS_PATH if docs_path == "infer" else validate_path(docs_path)
 
     docs_source_path = Path(docs_path).resolve() / "source"
 
@@ -440,10 +532,11 @@ def sphinx_docs_regenerate(
             except Exception:
                 pass
 
-    apidoc_command = f"sphinx-apidoc -f -e -o source {paths.APP_PATH.as_posix()}"
+    apidoc_command = (
+        f"sphinx-apidoc -f -e -o source {misc.get_console_str_with_quotes(PROJECT_PATHS.APP_PATH)}"
+    )
     subprocess.run(
         apidoc_command,
-        shell=True,
         cwd=docs_path,
         check=True,
     )
@@ -453,27 +546,29 @@ def sphinx_docs_regenerate(
             (docs_source_path / i).unlink()
 
     if build_locally:
-        subprocess.run(["make", "html"], shell=True, cwd=docs_path, check=True)
+        subprocess.run(["make", "html"], cwd=docs_path, check=True)
 
     if git_add:
-        subprocess.run(["git", "add", "docs"], shell=True, cwd=paths.ROOT_PATH, check=True)
+        subprocess.run(
+            ["git", "add", "docs"],
+            cwd=PROJECT_PATHS.ROOT_PATH.as_posix(),
+            check=True,
+        )
 
 
-def generate_readme_from_init(git_add=True):
+def generate_readme_from_init(git_add: bool = True) -> None:
     """Because i had very similar things in main __init__.py and in readme. It was to maintain news
     in code. For better simplicity i prefer write docs once and then generate. One code, two use cases.
 
     Why __init__? - Because in IDE on mouseover developers can see help.
     Why README.md? - Good for github.com
 
-    If issuing problems, try paths.set_root() to library path.
-
     Args:
         git_add (bool, optional): Whether to add generated files to stage. False mostly
             for testing reasons. Defaults to True.
     """
 
-    with open(paths.INIT_PATH) as fd:
+    with open(PROJECT_PATHS.INIT_PATH.as_posix()) as fd:
         file_contents = fd.read()
     module = ast.parse(file_contents)
     docstrings = ast.get_docstring(module)
@@ -481,8 +576,16 @@ def generate_readme_from_init(git_add=True):
     if docstrings is None:
         docstrings = ""
 
-    with open(paths.ROOT_PATH / "README.md", "w") as file:
+    with open(PROJECT_PATHS.README_PATH.as_posix(), "w") as file:
         file.write(docstrings)
 
     if git_add:
-        subprocess.run(["git", "add", "README.md"], shell=True, cwd=paths.ROOT_PATH, check=True)
+        subprocess.run(
+            [
+                "git",
+                "add",
+                PROJECT_PATHS.README_PATH,
+            ],
+            cwd=PROJECT_PATHS.ROOT_PATH.as_posix(),
+            check=True,
+        )
